@@ -1,5 +1,5 @@
 /* BufferedImage.java --
-   Copyright (C) 2000, 2002, 2003, 2004, 2005  Free Software Foundation
+   Copyright (C) 2000, 2002, 2003, 2004, 2005, 2006,  Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -80,7 +80,7 @@ public class BufferedImage extends Image
                           TYPE_BYTE_INDEXED   = 13;
   
   static final int[] bits3 = { 8, 8, 8 };
-  static final int[] bits4 = { 8, 8, 8 };
+  static final int[] bits4 = { 8, 8, 8, 8 };
   static final int[] bits1byte = { 8 };
   static final int[] bits1ushort = { 16 };
   
@@ -99,6 +99,35 @@ public class BufferedImage extends Image
 
   Vector observers;
   
+  /**
+   * Creates a new <code>BufferedImage</code> with the specified width, height
+   * and type.  Valid <code>type</code> values are:
+   * 
+   * <ul>
+   *   <li>{@link #TYPE_INT_RGB}</li>
+   *   <li>{@link #TYPE_INT_ARGB}</li>
+   *   <li>{@link #TYPE_INT_ARGB_PRE}</li>
+   *   <li>{@link #TYPE_INT_BGR}</li>
+   *   <li>{@link #TYPE_3BYTE_BGR}</li>
+   *   <li>{@link #TYPE_4BYTE_ABGR}</li>
+   *   <li>{@link #TYPE_4BYTE_ABGR_PRE}</li>
+   *   <li>{@link #TYPE_USHORT_565_RGB}</li>
+   *   <li>{@link #TYPE_USHORT_555_RGB}</li>
+   *   <li>{@link #TYPE_BYTE_GRAY}</li>
+   *   <li>{@link #TYPE_USHORT_GRAY}</li>
+   *   <li>{@link #TYPE_BYTE_BINARY}</li>
+   *   <li>{@link #TYPE_BYTE_INDEXED}</li>
+   * </ul>
+   * 
+   * @param w  the width (must be > 0).
+   * @param h  the height (must be > 0).
+   * @param type  the image type (see the list of valid types above).
+   * 
+   * @throws IllegalArgumentException if <code>w</code> or <code>h</code> is
+   *     less than or equal to zero.
+   * @throws IllegalArgumentException if <code>type</code> is not one of the
+   *     specified values.
+   */
   public BufferedImage(int w, int h, int type)
   {
     ColorModel cm = null;
@@ -174,13 +203,15 @@ public class BufferedImage extends Image
 	case TYPE_4BYTE_ABGR_PRE:
 	  bits = bits4;
 	  break;
-	case TYPE_BYTE_GRAY:
-	  bits = bits1byte;
-	  break;
-	case TYPE_USHORT_GRAY:
-	  bits = bits1ushort;
-	  dataType = DataBuffer.TYPE_USHORT;
-	  break;
+        case TYPE_BYTE_GRAY:
+          bits = bits1byte;
+          cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
+          break;
+        case TYPE_USHORT_GRAY:
+          bits = bits1ushort;
+          cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
+          dataType = DataBuffer.TYPE_USHORT;
+          break;
 	}
 	cm = new ComponentColorModel(cs, bits, alpha, premultiplied,
 				     alpha ?
@@ -196,6 +227,8 @@ public class BufferedImage extends Image
 	String msg2 = "type not implemented yet";
 	throw new UnsupportedOperationException(msg2);
 	// FIXME: build color-cube and create color model
+      default:
+        throw new IllegalArgumentException("Unknown image type " + type);
       }
     
     init(cm,
@@ -363,11 +396,28 @@ public class BufferedImage extends Image
 	return 1;
   }
 
+  /**
+   * Returns the value of the specified property, or 
+   * {@link Image#UndefinedProperty} if the property is not defined.
+   * 
+   * @param string  the property key (<code>null</code> not permitted).
+   * 
+   * @return The property value.
+   * 
+   * @throws NullPointerException if <code>string</code> is <code>null</code>.
+   */
   public Object getProperty(String string)
   {
-    if (properties == null)
-      return null;
-    return properties.get(string);
+    if (string == null)
+      throw new NullPointerException("The property name cannot be null.");
+    Object result = Image.UndefinedProperty;
+    if (properties != null)
+      {
+        Object v = properties.get(string);
+        if (v != null)
+          result = v;
+      }
+    return result;
   }
 
   public Object getProperty(String string, ImageObserver imageobserver)
@@ -375,10 +425,15 @@ public class BufferedImage extends Image
     return getProperty(string);
   }
 
-  
+  /**
+   * Returns <code>null</code> always.
+   * 
+   * @return <code>null</code> always.
+   */
   public String[] getPropertyNames()
   {
-    // FIXME: implement
+    // This method should always return null, see:
+    // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4640609
     return null;
   }
 
@@ -475,7 +530,10 @@ public class BufferedImage extends Image
           int[] pixels = getRGB(x, y, 
                                 width, height, 
                                 (int[])null, offset, stride);
-          ColorModel model = getColorModel();
+          // We already convert the color to RGB in the getRGB call, so
+          // we pass a simple RGB color model to the consumers.
+          ColorModel model = new DirectColorModel(32, 0xff0000, 0xff00, 0xff,
+                                                  0xff000000);
 
           consumers.add(ic);
 
