@@ -1,5 +1,5 @@
 /* Class.java -- Representation of a Java class.
-   Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004, 2005
+   Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation
 
 This file is part of GNU Classpath.
@@ -104,6 +104,23 @@ public final class Class<T>
    */
   private static final long serialVersionUID = 3206093459760846163L;
 
+  /**
+   * Flag indicating a synthetic member.
+   * Note that this duplicates a constant in Modifier.
+   */
+  private static final int SYNTHETIC = 0x1000;
+
+  /**
+   * Flag indiciating an annotation class.
+   */
+  private static final int ANNOTATION = 0x2000;
+
+  /**
+   * Flag indicating an enum constant or an enum class.
+   * Note that this duplicates a constant in Modifier.
+   */
+  private static final int ENUM = 0x4000;
+
   /** The class signers. */
   private Object[] signers = null;
   /** The class protection domain. */
@@ -265,7 +282,7 @@ public final class Class<T>
     ClassLoader loader = VMClass.getClassLoader(this);
     // Check if we may get the classloader
     SecurityManager sm = SecurityManager.current;
-    if (sm != null)
+    if (loader != null && sm != null)
       {
         // Get the calling classloader
 	ClassLoader cl = VMStackWalker.getCallingClassLoader();
@@ -1139,7 +1156,7 @@ public final class Class<T>
       }
     try
       {
-        return constructor.newInstance();
+        return constructor.newInstance(null);
       }
     catch (InvocationTargetException e)
       {
@@ -1302,7 +1319,9 @@ public final class Class<T>
    */
   public T cast(Object obj)
   {
-    return (T)VMClass.cast(obj, this);
+    if (obj != null && ! isInstance(obj))
+      throw new ClassCastException();
+    return (T) obj;
   }
 
   /**
@@ -1372,7 +1391,31 @@ public final class Class<T>
    */
   public T[] getEnumConstants()
   {
-    return (T[])VMClass.getEnumConstants(this);
+    if (isEnum())
+      {
+	try
+	  {
+	    return (T[]) getMethod("values").invoke(null);
+	  }
+	catch (NoSuchMethodException exception)
+	  {
+	    throw new Error("Enum lacks values() method");
+	  }
+	catch (IllegalAccessException exception)
+	  {
+	    throw new Error("Unable to access Enum class");
+	  }
+	catch (InvocationTargetException exception)
+	  {
+	    throw new
+	      RuntimeException("The values method threw an exception",
+			       exception);
+	  }
+      }
+    else
+      {
+	return null;
+      }
   }
 
   /**
@@ -1383,7 +1426,8 @@ public final class Class<T>
    */
   public boolean isEnum()
   {
-    return VMClass.isEnum(this);
+    int mod = VMClass.getModifiers (this, true);
+    return (mod & ENUM) != 0;
   }
 
   /**
@@ -1395,7 +1439,8 @@ public final class Class<T>
    */
   public boolean isSynthetic()
   {
-    return VMClass.isSynthetic(this);
+    int mod = VMClass.getModifiers (this, true);
+    return (mod & SYNTHETIC) != 0;
   }
 
   /**
@@ -1406,7 +1451,8 @@ public final class Class<T>
    */
   public boolean isAnnotation()
   {
-    return VMClass.isAnnotation(this);
+    int mod = VMClass.getModifiers (this, true);
+    return (mod & ANNOTATION) != 0;
   }
 
   /**
