@@ -484,7 +484,7 @@ public abstract class NumberFormat extends Format implements Cloneable
    */
   public static NumberFormat getIntegerInstance(Locale locale)
   {
-    NumberFormat format = computeInstance (locale, "numberFormat", "#,##0");
+    NumberFormat format = computeInstance (locale, "integerFormat", "#,##0");
     format.setMaximumFractionDigits(0);
     format.setParseIntegerOnly (true);
     return format;
