@@ -39,15 +39,19 @@ exception statement from your version. */
 package javax.swing.text.html;
 
 import gnu.classpath.NotImplementedException;
+import gnu.javax.swing.text.html.CharacterAttributeTranslator;
+import gnu.javax.swing.text.html.parser.htmlAttributeSet;
 
 import java.io.IOException;
+import java.io.StringReader;
 import java.net.URL;
 import java.util.HashMap;
 import java.util.Stack;
 import java.util.Vector;
 
+import javax.swing.JEditorPane;
 import javax.swing.event.DocumentEvent;
-import javax.swing.event.UndoableEditEvent;
+import javax.swing.event.HyperlinkEvent.EventType;
 import javax.swing.text.AbstractDocument;
 import javax.swing.text.AttributeSet;
 import javax.swing.text.BadLocationException;
@@ -61,9 +65,15 @@ import javax.swing.text.StyleConstants;
 import javax.swing.text.html.HTML.Tag;
 
 /**
- * TODO: Add more comments here 
+ * Represents the HTML document that is constructed by defining the text and
+ * other components (images, buttons, etc) in HTML language. This class can
+ * becomes the default document for {@link JEditorPane} after setting its
+ * content type to "text/html". HTML document also serves as an intermediate
+ * data structure when it is needed to parse HTML and then obtain the content of
+ * the certain types of tags. This class also has methods for modifying the HTML
+ * content.
  * 
- * @author Audrius Meskauskas, Lithuania (AudriusA@Bioinformatics.org)
+ * @author Audrius Meskauskas (AudriusA@Bioinformatics.org)
  * @author Anthony Balkissoon (abalkiss@redhat.com)
  * @author Lillian Angel (langel@redhat.com)
  */
@@ -130,21 +140,6 @@ public class HTMLDocument extends DefaultStyledDocument
     return styleSheet;
   }
   
-  /**
-   * Replaces the contents of the document with the given element
-   * specifications. This is called before insert if the loading is done
-   * in bursts. This is the only method called if loading the document
-   * entirely in one burst.
-   * 
-   * @param data - the date that replaces the content of the document
-   */
-  protected void create(DefaultStyledDocument.ElementSpec[] data)
-  {
-    // Once the super behaviour is properly implemented it should be sufficient
-    // to simply call super.create(data).
-    super.create(data);
-  }
-  
   /**
    * This method creates a root element for the new document.
    * 
@@ -215,41 +210,6 @@ public class HTMLDocument extends DefaultStyledDocument
     return new BlockElement(parent, a);
   }
   
-  /**
-   * Inserts new elements in bulk. This is how elements get created in the
-   * document. The parsing determines what structure is needed and creates the
-   * specification as a set of tokens that describe the edit while leaving the
-   * document free of a write-lock. This method can then be called in bursts by
-   * the reader to acquire a write-lock for a shorter duration (i.e. while the
-   * document is actually being altered). 
-   * 
-   * @param offset - the starting offset 
-   * @param data - the element data
-   * @throws BadLocationException - if the given position does not
-   * represent a valid location in the associated document.
-   */
-  protected void insert(int offset, DefaultStyledDocument.ElementSpec[] data)
-    throws BadLocationException
-  {
-    super.insert(offset, data);
-  }
-  
-  /**
-   * Updates document structure as a result of text insertion. This will happen
-   * within a write lock. This implementation simply parses the inserted content
-   * for line breaks and builds up a set of instructions for the element buffer.
-   * 
-   * @param chng - a description of the document change
-   * @param attr - the attributes
-   */
-  protected void insertUpdate(AbstractDocument.DefaultDocumentEvent chng, 
-                              AttributeSet attr)
-  {
-    // FIXME: Not implemented
-    System.out.println("insertUpdate not implemented");
-    super.insertUpdate(chng, attr);    
-  }
-  
   /**
    * Returns the parser used by this HTMLDocument to insert HTML.
    * 
@@ -414,6 +374,7 @@ public class HTMLDocument extends DefaultStyledDocument
   }
 
   public void processHTMLFrameHyperlinkEvent(HTMLFrameHyperlinkEvent event)
+  throws NotImplementedException
   {
     // TODO: Implement this properly.
   }
@@ -558,19 +519,23 @@ public class HTMLDocument extends DefaultStyledDocument
    */
   public class HTMLReader extends HTMLEditorKit.ParserCallback
   {    
-    /** Holds the current character attribute set **/
+    /**
+     * Holds the current character attribute set *
+     */
     protected MutableAttributeSet charAttr = new SimpleAttributeSet();
     
     protected Vector<ElementSpec> parseBuffer = new Vector<ElementSpec>();
     
-    /** A stack for character attribute sets **/
+    /** 
+     * A stack for character attribute sets *
+     */
     Stack charAttrStack = new Stack();
 
     /**
      * The parse stack. This stack holds HTML.Tag objects that reflect the
      * current position in the parsing process.
      */
-    private Stack parseStack = new Stack();
+    Stack parseStack = new Stack();
    
     /** A mapping between HTML.Tag objects and the actions that handle them **/
     HashMap tagToAction;
@@ -578,10 +543,31 @@ public class HTMLDocument extends DefaultStyledDocument
     /** Tells us whether we've received the '</html>' tag yet **/
     boolean endHTMLEncountered = false;
     
-    /** Variables related to the constructor with explicit insertTag **/
-    int popDepth, pushDepth, offset;
+    /** 
+     * Related to the constructor with explicit insertTag 
+     */
+    int popDepth;
+    
+    /**
+     * Related to the constructor with explicit insertTag
+     */    
+    int pushDepth;
+    
+    /** 
+     * Related to the constructor with explicit insertTag
+     */    
+    int offset;
+    
+    /**
+     * The tag (inclusve), after that the insertion should start.
+     */
     HTML.Tag insertTag;
-    boolean insertTagEncountered = false;
+    
+    /**
+     * This variable becomes true after the insert tag has been encountered.
+     */
+    boolean insertTagEncountered;
+
     
     /** A temporary variable that helps with the printing out of debug information **/
     boolean debug = false;
@@ -646,12 +632,16 @@ public class HTMLDocument extends DefaultStyledDocument
       {
         // Put the old attribute set on the stack.
         pushCharacterStyle();
-        
-        // And create the new one by adding the attributes in <code>a</code>.
-        if (a != null)
-          charAttr.addAttribute(t, a.copyAttributes());          
+
+	// Translate tag.. return if succesful.
+	if(CharacterAttributeTranslator.translateTag(charAttr, t, a))
+	  return;
+
+        // Just add the attributes in <code>a</code>.
+ 	if (a != null)
+ 	  charAttr.addAttribute(t, a.copyAttributes());          
       }
-      
+
       /**
        * Called when an end tag is seen for one of the types of tags associated
        * with this Action.
@@ -687,17 +677,25 @@ public class HTMLDocument extends DefaultStyledDocument
       } 
     }
     
-    public class HiddenAction extends TagAction
+    /**
+     * This action indicates that the content between starting and closing HTML
+     * elements (like script - /script) should not be visible. The content is
+     * still inserted and can be accessed when iterating the HTML document. The
+     * parser will only fire
+     * {@link javax.swing.text.html.HTMLEditorKit.ParserCallback#handleText} for
+     * the hidden tags, regardless from that html tags the hidden section may
+     * contain.
+     */
+    public class HiddenAction
+        extends TagAction
     {
       /**
        * This method is called when a start tag is seen for one of the types
        * of tags associated with this Action.
        */
       public void start(HTML.Tag t, MutableAttributeSet a)
-        throws NotImplementedException
       {
-        // FIXME: Implement.
-        print ("HiddenAction.start not implemented");
+        blockOpen(t, a);
       }
       
       /**
@@ -705,10 +703,8 @@ public class HTMLDocument extends DefaultStyledDocument
        * with this Action.
        */
       public void end(HTML.Tag t)
-        throws NotImplementedException
       {
-        // FIXME: Implement.
-        print ("HiddenAction.end not implemented");
+        blockClose(t);
       } 
     }
     
@@ -724,17 +720,6 @@ public class HTMLDocument extends DefaultStyledDocument
         // FIXME: Implement.
         print ("IsindexAction.start not implemented");
       }
-      
-      /**
-       * Called when an end tag is seen for one of the types of tags associated
-       * with this Action.
-       */
-      public void end(HTML.Tag t)
-        throws NotImplementedException
-      {
-        // FIXME: Implement.
-        print ("IsindexAction.end not implemented");
-      } 
     }
     
     public class ParagraphAction extends BlockAction
@@ -745,7 +730,6 @@ public class HTMLDocument extends DefaultStyledDocument
        */
       public void start(HTML.Tag t, MutableAttributeSet a)
       {
-        // FIXME: What else must be done here?
         blockOpen(t, a);
       }
       
@@ -755,7 +739,6 @@ public class HTMLDocument extends DefaultStyledDocument
        */
       public void end(HTML.Tag t)
       {
-        // FIXME: What else must be done here?
         blockClose(t);
       } 
     }
@@ -771,6 +754,7 @@ public class HTMLDocument extends DefaultStyledDocument
       {
         // FIXME: Implement.
         print ("PreAction.start not implemented");
+        super.start(t, a);
       }
       
       /**
@@ -782,32 +766,26 @@ public class HTMLDocument extends DefaultStyledDocument
       {
         // FIXME: Implement.
         print ("PreAction.end not implemented");
+        super.end(t);
       } 
     }
     
+    /**
+     * Inserts the elements that are represented by ths single tag with 
+     * attributes (only). The closing tag, even if present, mut follow
+     * immediately after the starting tag without providing any additional
+     * information. Hence the {@link TagAction#end} method need not be
+     * overridden and still does nothing.
+     */
     public class SpecialAction extends TagAction
     {
       /**
-       * This method is called when a start tag is seen for one of the types
-       * of tags associated with this Action.
+       * The functionality is delegated to {@link HTMLReader#addSpecialElement}
        */
       public void start(HTML.Tag t, MutableAttributeSet a)
-        throws NotImplementedException
       {
-        // FIXME: Implement.
-        print ("SpecialAction.start not implemented");
+        addSpecialElement(t, a);
       }
-      
-      /**
-       * Called when an end tag is seen for one of the types of tags associated
-       * with this Action.
-       */
-      public void end(HTML.Tag t)
-        throws NotImplementedException
-      {
-        // FIXME: Implement.
-        print ("SpecialAction.end not implemented");
-      }                
     }
     
     class AreaAction extends TagAction
@@ -1131,7 +1109,7 @@ public class HTMLDocument extends DefaultStyledDocument
      */
     protected void pushCharacterStyle()
     {
-      charAttrStack.push(charAttr);
+      charAttrStack.push(charAttr.copyAttributes());
     }
     
     /**
@@ -1190,8 +1168,21 @@ public class HTMLDocument extends DefaultStyledDocument
     }
     
     /**
-     * This method is called by the parser and should route the call to 
-     * the proper handler for the tag.
+     * Checks if the HTML tag should be inserted. The tags before insert tag (if
+     * specified) are not inserted. Also, the tags after the end of the html are
+     * not inserted.
+     * 
+     * @return true if the tag should be inserted, false otherwise.
+     */
+    private boolean shouldInsert()
+    {
+      return ! endHTMLEncountered
+             && (insertTagEncountered || insertTag == null);
+    }
+    
+    /**
+     * This method is called by the parser and should route the call to the
+     * proper handler for the tag.
      * 
      * @param t the HTML.Tag
      * @param a the attribute set
@@ -1199,13 +1190,15 @@ public class HTMLDocument extends DefaultStyledDocument
      */
     public void handleStartTag(HTML.Tag t, MutableAttributeSet a, int pos)
     {
-      // Don't call the Action if we've already seen </html>.
-      if (endHTMLEncountered)
-        return;
-        
-      TagAction action = (TagAction) tagToAction.get(t);
-      if (action != null)
-        action.start(t, a);      
+      if (t == insertTag)
+        insertTagEncountered = true;
+
+      if (shouldInsert())
+        {
+          TagAction action = (TagAction) tagToAction.get(t);
+          if (action != null)
+            action.start(t, a);
+        }
     }
     
     /**
@@ -1216,42 +1209,41 @@ public class HTMLDocument extends DefaultStyledDocument
      */
     public void handleComment(char[] data, int pos)
     {
-      // Don't call the Action if we've already seen </html>.
-      if (endHTMLEncountered)
-        return;
-      
-      TagAction action = (TagAction) tagToAction.get(HTML.Tag.COMMENT);
-      if (action != null)
+      if (shouldInsert())
         {
-          action.start(HTML.Tag.COMMENT, new SimpleAttributeSet());
-          action.end (HTML.Tag.COMMENT);
+          TagAction action = (TagAction) tagToAction.get(HTML.Tag.COMMENT);
+          if (action != null)
+            {
+              action.start(HTML.Tag.COMMENT, 
+                           htmlAttributeSet.EMPTY_HTML_ATTRIBUTE_SET);
+              action.end(HTML.Tag.COMMENT);
+            }
         }
     }
     
     /**
-     * This method is called by the parser and should route the call to 
-     * the proper handler for the tag.
+     * This method is called by the parser and should route the call to the
+     * proper handler for the tag.
      * 
      * @param t the HTML.Tag
      * @param pos the position at which the tag was encountered
      */
     public void handleEndTag(HTML.Tag t, int pos)
     {
-      // Don't call the Action if we've already seen </html>.
-      if (endHTMLEncountered)
-        return;
-      
-      // If this is the </html> tag we need to stop calling the Actions
-      if (t == HTML.Tag.HTML)
-        endHTMLEncountered = true;
-      
-      TagAction action = (TagAction) tagToAction.get(t);
-      if (action != null)
-        action.end(t);
+      if (shouldInsert())
+        {
+          // If this is the </html> tag we need to stop calling the Actions
+          if (t == HTML.Tag.HTML)
+            endHTMLEncountered = true;
+
+          TagAction action = (TagAction) tagToAction.get(t);
+          if (action != null)
+            action.end(t);
+        }
     }
     
     /**
-     * This is a callback from the parser that should be routed to the 
+     * This is a callback from the parser that should be routed to the
      * appropriate handler for the tag.
      * 
      * @param t the HTML.Tag that was encountered
@@ -1260,15 +1252,17 @@ public class HTMLDocument extends DefaultStyledDocument
      */
     public void handleSimpleTag(HTML.Tag t, MutableAttributeSet a, int pos)
     {
-      // Don't call the Action if we've already seen </html>.
-      if (endHTMLEncountered)
-        return;
-      
-      TagAction action = (TagAction) tagToAction.get (t);
-      if (action != null)
+      if (t == insertTag)
+        insertTagEncountered = true;
+
+      if (shouldInsert())
         {
-          action.start(t, a);
-          action.end(t);
+          TagAction action = (TagAction) tagToAction.get(t);
+          if (action != null)
+            {
+              action.start(t, a);
+              action.end(t);
+            }
         }
     }
     
@@ -1281,7 +1275,6 @@ public class HTMLDocument extends DefaultStyledDocument
      * @since 1.3
      */
     public void handleEndOfLineString(String eol)
-      throws NotImplementedException
     {
       // FIXME: Implement.
       print ("HTMLReader.handleEndOfLineString not implemented yet");
@@ -1324,16 +1317,6 @@ public class HTMLDocument extends DefaultStyledDocument
       printBuffer();
       DefaultStyledDocument.ElementSpec element;
 
-      // If the previous tag is content and the parent is p-implied, then
-      // we must also close the p-implied.
-      if (parseStack.size() > 0 && parseStack.peek() == HTML.Tag.IMPLIED)
-        {
-          element = new DefaultStyledDocument.ElementSpec(null,
-                                    DefaultStyledDocument.ElementSpec.EndTagType);
-          parseBuffer.addElement(element);
-          parseStack.pop();
-        }
-
       parseStack.push(t);
       AbstractDocument.AttributeContext ctx = getAttributeContext();
       AttributeSet copy = attr.copyAttributes();
@@ -1371,16 +1354,6 @@ public class HTMLDocument extends DefaultStyledDocument
                                     new char[0], 0, 0);
           parseBuffer.add(element);
         }
-      // If the previous tag is content and the parent is p-implied, then
-      // we must also close the p-implied.
-      else if (parseStack.peek() == HTML.Tag.IMPLIED)
-        {
-          element = new DefaultStyledDocument.ElementSpec(null,
-                                 DefaultStyledDocument.ElementSpec.EndTagType);
-          parseBuffer.addElement(element);
-          if (parseStack.size() > 0)
-            parseStack.pop();
-        }
 
       element = new DefaultStyledDocument.ElementSpec(null,
 				DefaultStyledDocument.ElementSpec.EndTagType);
@@ -1420,27 +1393,6 @@ public class HTMLDocument extends DefaultStyledDocument
       DefaultStyledDocument.ElementSpec element;
       AttributeSet attributes = null;
 
-      // Content must always be embedded inside a paragraph element,
-      // so we create this if the previous element is not one of
-      // <p>, <h1> .. <h6>.
-      boolean createImpliedParagraph = false;
-      HTML.Tag parent = (HTML.Tag) parseStack.peek();
-      if (parent != HTML.Tag.P && parent != HTML.Tag.H1
-          && parent != HTML.Tag.H2
-          && parent != HTML.Tag.H3 && parent != HTML.Tag.H4
-          && parent != HTML.Tag.H5 && parent != HTML.Tag.H6
-          && parent != HTML.Tag.TD)
-        {
-          attributes = ctx.getEmptySet();
-          attributes = ctx.addAttribute(attributes,
-                                        StyleConstants.NameAttribute,
-                                        HTML.Tag.IMPLIED);
-          element = new DefaultStyledDocument.ElementSpec(attributes,
-                       DefaultStyledDocument.ElementSpec.StartTagType);
-          parseBuffer.add(element);
-          parseStack.push(HTML.Tag.IMPLIED);
-        }
-
       // Copy the attribute set, don't use the same object because 
       // it may change
       if (charAttr != null)
@@ -1478,10 +1430,21 @@ public class HTMLDocument extends DefaultStyledDocument
      * @param a the attribute set specifying the special content
      */
     protected void addSpecialElement(HTML.Tag t, MutableAttributeSet a)
-      throws NotImplementedException
     {
-      // FIXME: Implement
-      print ("HTMLReader.addSpecialElement not implemented yet");
+      a.addAttribute(StyleConstants.NameAttribute, t);
+      
+      // Migrate from the rather htmlAttributeSet to the faster, lighter and 
+      // unchangeable alternative implementation.
+      AttributeSet copy = a.copyAttributes();
+
+      // The two spaces are required because some special elements like HR
+      // must be broken. At least two characters are needed to break into the
+      // two parts.
+      DefaultStyledDocument.ElementSpec spec =
+        new DefaultStyledDocument.ElementSpec(copy,
+	  DefaultStyledDocument.ElementSpec.ContentType, 
+          new char[] {' ', ' '}, 0, 2 );
+      parseBuffer.add(spec);
     }
     
     void printBuffer()
@@ -1521,7 +1484,61 @@ public class HTMLDocument extends DefaultStyledDocument
                                                 HTML.Tag insertTag)
   {
     return new HTMLReader(pos, popDepth, pushDepth, insertTag);
-  }  
+  }
+  
+  /**
+   * Gets the reader for the parser to use when inserting the HTML fragment into
+   * the document. Checks if the parser is present, sets the parent in the
+   * element stack and removes any actions for BODY (it can be only one body in
+   * a HTMLDocument).
+   * 
+   * @param pos - the starting position
+   * @param popDepth - the number of EndTagTypes to generate before inserting
+   * @param pushDepth - the number of StartTagTypes with a direction of
+   *          JoinNextDirection that should be generated before inserting, but
+   *          after the end tags have been generated.
+   * @param insertTag - the first tag to start inserting into document
+   * @param parent the element that will be the parent in the document. HTML
+   *          parsing includes checks for the parent, so it must be available.
+   * @return - the reader
+   * @throws IllegalStateException if the parsert is not set.
+   */
+  public HTMLEditorKit.ParserCallback getInsertingReader(int pos, int popDepth,
+                                                         int pushDepth,
+                                                         HTML.Tag insertTag,
+                                                         final Element parent)
+      throws IllegalStateException
+  {
+    if (parser == null)
+      throw new IllegalStateException("Parser has not been set");
+
+    HTMLReader reader = new HTMLReader(pos, popDepth, pushDepth, insertTag)
+    {
+      /**
+       * Ignore BODY.
+       */
+      public void handleStartTag(HTML.Tag t, MutableAttributeSet a, int pos)
+      {
+        if (t != HTML.Tag.BODY)
+          super.handleStartTag(t, a, pos);
+      }
+
+      /**
+       * Ignore BODY.
+       */
+      public void handleEndTag(HTML.Tag t, int pos)
+      {
+        if (t != HTML.Tag.BODY)
+          super.handleEndTag(t, pos);
+      }
+    };
+      
+    // Set the parent HTML tag.
+    reader.parseStack.push(parent.getAttributes().getAttribute(
+      StyleConstants.NameAttribute));
+
+    return reader;
+  }   
   
   /**
    * Gets the child element that contains the attribute with the value or null.
@@ -1530,8 +1547,8 @@ public class HTMLDocument extends DefaultStyledDocument
    * @param e - the element to begin search at
    * @param attribute - the desired attribute
    * @param value - the desired value
-   * @return the element found with the attribute and value specified or null
-   * if it is not found.
+   * @return the element found with the attribute and value specified or null if
+   *         it is not found.
    */
   public Element getElement(Element e, Object attribute, Object value)
   {
@@ -1556,17 +1573,18 @@ public class HTMLDocument extends DefaultStyledDocument
   }
   
   /**
-   * Returns the element that has the given id Attribute. If it is not found, 
-   * null is returned. This method works on an Attribute, not a character tag.
-   * This is not thread-safe.
+   * Returns the element that has the given id Attribute (for instance, &lt;p id
+   * ='my paragraph &gt;'). If it is not found, null is returned. The HTML tag,
+   * having this attribute, is not checked by this method and can be any. The
+   * method is not thread-safe.
    * 
-   * @param attrId - the Attribute id to look for
+   * @param attrId - the value of the attribute id to look for
    * @return the element that has the given id.
    */
   public Element getElement(String attrId)
   {
-    Element root = getDefaultRootElement();
-    return getElement(root, HTML.getAttributeKey(attrId) , attrId);
+    return getElement(getDefaultRootElement(), HTML.Attribute.ID,
+                      attrId);
   }
   
   /**
@@ -1582,22 +1600,30 @@ public class HTMLDocument extends DefaultStyledDocument
    * @throws IllegalStateException - if an HTMLEditorKit.Parser has not been set
    */
   public void setInnerHTML(Element elem, String htmlText) 
-    throws BadLocationException, IOException, NotImplementedException
+    throws BadLocationException, IOException
   {
     if (elem.isLeaf())
       throw new IllegalArgumentException("Element is a leaf");
-    if (parser == null)
-      throw new IllegalStateException("Parser has not been set");
-    // FIXME: Not implemented fully, use InsertHTML* in HTMLEditorKit?
-    System.out.println("setInnerHTML not implemented");
+    
+    int start = elem.getStartOffset();
+    int end = elem.getEndOffset();
+
+    HTMLEditorKit.ParserCallback reader = getInsertingReader(
+      end, 0, 0, HTML.Tag.BODY, elem);
+
+    // TODO charset
+    getParser().parse(new StringReader(htmlText), reader, true);
+    
+    // Remove the previous content
+    remove(start, end - start);
   }
   
   /**
-   * Replaces the given element in the parent with the string. When replacing
-   * a leaf, this will attempt to make sure there is a newline present if one is
-   * needed. This may result in an additional element being inserted.
-   * This will be seen as at least two events, n inserts followed by a remove.
-   * The HTMLEditorKit.Parser must be set.
+   * Replaces the given element in the parent with the string. When replacing a
+   * leaf, this will attempt to make sure there is a newline present if one is
+   * needed. This may result in an additional element being inserted. This will
+   * be seen as at least two events, n inserts followed by a remove. The
+   * HTMLEditorKit.Parser must be set.
    * 
    * @param elem - the branch element whose parent will be replaced
    * @param htmlText - the string to be parsed and assigned to elem
@@ -1605,18 +1631,25 @@ public class HTMLDocument extends DefaultStyledDocument
    * @throws IOException
    * @throws IllegalStateException - if parser is not set
    */
-  public void setOuterHTML(Element elem, String htmlText) 
-    throws BadLocationException, IOException, NotImplementedException
-    {
-      if (parser == null)
-        throw new IllegalStateException("Parser has not been set");
-      // FIXME: Not implemented fully, use InsertHTML* in HTMLEditorKit?
-      System.out.println("setOuterHTML not implemented");
-    }
+public void setOuterHTML(Element elem, String htmlText)
+      throws BadLocationException, IOException
+  {
+    // Remove the current element:
+    int start = elem.getStartOffset();
+    int end = elem.getEndOffset();
+
+    remove(start, end-start);
+       
+    HTMLEditorKit.ParserCallback reader = getInsertingReader(
+      start, 0, 0, HTML.Tag.BODY, elem);
+
+    // TODO charset
+    getParser().parse(new StringReader(htmlText), reader, true);
+  }
   
   /**
-   * Inserts the string before the start of the given element.
-   * The parser must be set.
+   * Inserts the string before the start of the given element. The parser must
+   * be set.
    * 
    * @param elem - the element to be the root for the new text.
    * @param htmlText - the string to be parsed and assigned to elem
@@ -1625,18 +1658,19 @@ public class HTMLDocument extends DefaultStyledDocument
    * @throws IllegalStateException - if parser has not been set
    */
   public void insertBeforeStart(Element elem, String htmlText)
-      throws BadLocationException, IOException, NotImplementedException
+      throws BadLocationException, IOException
   {
-    if (parser == null)
-      throw new IllegalStateException("Parser has not been set");
-    //  FIXME: Not implemented fully, use InsertHTML* in HTMLEditorKit?
-    System.out.println("insertBeforeStart not implemented");
+    HTMLEditorKit.ParserCallback reader = getInsertingReader(
+      elem.getStartOffset(), 0, 0, HTML.Tag.BODY, elem);
+
+    // TODO charset
+    getParser().parse(new StringReader(htmlText), reader, true);
   }
   
   /**
-   * Inserts the string at the end of the element. If elem's children
-   * are leaves, and the character at elem.getEndOffset() - 1 is a newline, 
-   * then it will be inserted before the newline. The parser must be set.
+   * Inserts the string at the end of the element. If elem's children are
+   * leaves, and the character at elem.getEndOffset() - 1 is a newline, then it
+   * will be inserted before the newline. The parser must be set.
    * 
    * @param elem - the element to be the root for the new text
    * @param htmlText - the text to insert
@@ -1645,12 +1679,14 @@ public class HTMLDocument extends DefaultStyledDocument
    * @throws IllegalStateException - if parser is not set
    */
   public void insertBeforeEnd(Element elem, String htmlText)
-      throws BadLocationException, IOException, NotImplementedException
+      throws BadLocationException, IOException
   {
-    if (parser == null)
-      throw new IllegalStateException("Parser has not been set");
-    //  FIXME: Not implemented fully, use InsertHTML* in HTMLEditorKit?
-    System.out.println("insertBeforeEnd not implemented");
+    HTMLEditorKit.ParserCallback reader = getInsertingReader(
+      elem.getEndOffset(), 0, 0, HTML.Tag.BODY, elem);
+
+    // TODO charset
+    getParser().parse(new StringReader(htmlText), reader, true);
+
   }
   
   /**
@@ -1664,12 +1700,13 @@ public class HTMLDocument extends DefaultStyledDocument
    * @throws IllegalStateException - if parser is not set
    */
   public void insertAfterEnd(Element elem, String htmlText)
-      throws BadLocationException, IOException, NotImplementedException
+      throws BadLocationException, IOException
   {
-    if (parser == null)
-      throw new IllegalStateException("Parser has not been set");
-    //  FIXME: Not implemented fully, use InsertHTML* in HTMLEditorKit?
-    System.out.println("insertAfterEnd not implemented");
+    HTMLEditorKit.ParserCallback reader = getInsertingReader(
+      elem.getEndOffset(), 0, 0, HTML.Tag.BODY, elem);
+
+    // TODO charset
+    getParser().parse(new StringReader(htmlText), reader, true);
   }
   
   /**
@@ -1683,60 +1720,12 @@ public class HTMLDocument extends DefaultStyledDocument
    * @throws IllegalStateException - if parser is not set
    */
   public void insertAfterStart(Element elem, String htmlText)
-      throws BadLocationException, IOException, NotImplementedException
+      throws BadLocationException, IOException
   {
-    if (parser == null)
-      throw new IllegalStateException("Parser has not been set");
-    //  FIXME: Not implemented fully, use InsertHTML* in HTMLEditorKit?
-    System.out.println("insertAfterStart not implemented");
-  }
-  
-  /**
-   * This method sets the attributes associated with the paragraph containing
-   * offset. If replace is false, s is merged with existing attributes. The
-   * length argument determines how many characters are affected by the new
-   * attributes. This is often the entire paragraph.
-   * 
-   * @param offset -
-   *          the offset into the paragraph (must be at least 0)
-   * @param length -
-   *          the number of characters affected (must be at least 0)
-   * @param s -
-   *          the attributes
-   * @param replace -
-   *          whether to replace existing attributes, or merge them
-   */
-  public void setParagraphAttributes(int offset, int length, AttributeSet s,
-                                     boolean replace)
-    throws NotImplementedException
-  {
-    //  FIXME: Not implemented.
-    System.out.println("setParagraphAttributes not implemented");
-    super.setParagraphAttributes(offset, length, s, replace);
-  }
-  
-  /**
-   * This method flags a change in the document.
-   * 
-   *  @param e - the Document event
-   */
-  protected void fireChangedUpdate(DocumentEvent e)
-    throws NotImplementedException
-  {
-    //  FIXME: Not implemented.
-    System.out.println("fireChangedUpdate not implemented");
-    super.fireChangedUpdate(e);    
-  }
+    HTMLEditorKit.ParserCallback reader = getInsertingReader(
+      elem.getStartOffset(), 0, 0, HTML.Tag.BODY, elem);
 
-  /**
-   * This method fires an event intended to be caught by Undo listeners. It
-   * simply calls the super version inherited from DefaultStyledDocument. With
-   * this method, an HTML editor could easily provide undo support.
-   * 
-   * @param e - the UndoableEditEvent
-   */
-  protected void fireUndoableEditUpdate(UndoableEditEvent e)
-  {
-    super.fireUndoableEditUpdate(e);
+    // TODO charset
+    getParser().parse(new StringReader(htmlText), reader, true);
   }
 }
