@@ -38,7 +38,10 @@ exception statement from your version. */
 
 package java.lang.reflect;
 
+import gnu.java.lang.ClassHelper;
+
 import gnu.java.lang.reflect.MethodSignatureParser;
+
 import java.util.Arrays;
 
 /**
@@ -81,6 +84,11 @@ extends AccessibleObject implements Member, GenericDeclaration
   String name;
   int slot;
 
+  private static final int METHOD_MODIFIERS
+    = Modifier.ABSTRACT | Modifier.FINAL | Modifier.NATIVE
+      | Modifier.PRIVATE | Modifier.PROTECTED | Modifier.PUBLIC
+      | Modifier.STATIC | Modifier.STRICT | Modifier.SYNCHRONIZED;
+
   /**
    * This class is uninstantiable.
    */
@@ -110,6 +118,12 @@ extends AccessibleObject implements Member, GenericDeclaration
     return name;
   }
 
+  /**
+   * Return the raw modifiers for this method.
+   * @return the method's modifiers
+   */
+  private native int getModifiersInternal();
+
   /**
    * Gets the modifiers this method uses.  Use the <code>Modifier</code>
    * class to interpret the values.  A method can only have a subset of the
@@ -119,7 +133,40 @@ extends AccessibleObject implements Member, GenericDeclaration
    * @return an integer representing the modifiers to this Member
    * @see Modifier
    */
-  public native int getModifiers();
+  public int getModifiers()
+  {
+    return getModifiersInternal() & METHOD_MODIFIERS;
+  }
+
+  /**
+   * Return true if this method is a bridge method.  A bridge method
+   * is generated by the compiler in some situations involving
+   * generics and inheritance.
+   * @since 1.5
+   */
+  public boolean isBridge()
+  {
+    return (getModifiersInternal() & Modifier.BRIDGE) != 0;
+  }
+
+  /**
+   * Return true if this method is synthetic, false otherwise.
+   * @since 1.5
+   */
+  public boolean isSynthetic()
+  {
+    return (getModifiersInternal() & Modifier.SYNTHETIC) != 0;
+  }
+
+  /**
+   * Return true if this is a varargs method, that is if
+   * the method takes a variable number of arguments.
+   * @since 1.5
+   */
+  public boolean isVarArgs()
+  {
+    return (getModifiersInternal() & Modifier.VARARGS) != 0;
+  }
 
   /**
    * Gets the return type of this method.
@@ -213,15 +260,15 @@ extends AccessibleObject implements Member, GenericDeclaration
     // 128 is a reasonable buffer initial size for constructor
     StringBuilder sb = new StringBuilder(128);
     Modifier.toString(getModifiers(), sb).append(' ');
-    sb.append(getUserTypeName(getReturnType().getName())).append(' ');
+    sb.append(ClassHelper.getUserName(getReturnType())).append(' ');
     sb.append(getDeclaringClass().getName()).append('.');
     sb.append(getName()).append('(');
     Class[] c = getParameterTypes();
     if (c.length > 0)
       {
-        sb.append(getUserTypeName(c[0].getName()));
+        sb.append(ClassHelper.getUserName(c[0]));
         for (int i = 1; i < c.length; i++)
-          sb.append(',').append(getUserTypeName(c[i].getName()));
+          sb.append(',').append(ClassHelper.getUserName(c[i]));
       }
     sb.append(')');
     c = getExceptionTypes();
@@ -234,53 +281,31 @@ extends AccessibleObject implements Member, GenericDeclaration
     return sb.toString();
   }
 
-  private static String getUserTypeName(String typeSpec)
+  public String toGenericString()
   {
-    int pos = 0;
-    String typeName = "";
-    String arrayPart = "";
-
-    while (typeSpec.charAt(pos) == '[')
+    // 128 is a reasonable buffer initial size for constructor
+    StringBuilder sb = new StringBuilder(128);
+    Modifier.toString(getModifiers(), sb).append(' ');
+    Constructor.addTypeParameters(sb, getTypeParameters());
+    sb.append(getGenericReturnType()).append(' ');
+    sb.append(getDeclaringClass().getName()).append('.');
+    sb.append(getName()).append('(');
+    Type[] types = getGenericParameterTypes();
+    if (types.length > 0)
       {
-	arrayPart += "[]";
-	++pos;
+        sb.append(types[0]);
+        for (int i = 1; i < types.length; i++)
+          sb.append(',').append(types[i]);
       }
-
-    switch (typeSpec.charAt(pos))
+    sb.append(')');
+    types = getGenericExceptionTypes();
+    if (types.length > 0)
       {
-      case 'Z':
-	typeName = "boolean";
-	break;
-      case 'B':
-	typeName = "byte";
-	break;
-      case 'C':
-	typeName = "char";
-	break;
-      case 'D':
-	typeName = "double";
-	break;
-      case 'F':
-	typeName = "float";
-	break;
-      case 'I':
-	typeName = "int";
-	break;
-      case 'J':
-	typeName = "long";
-	break;
-      case 'S':
-	typeName = "short";
-	break;
-      case 'L':
-	typeName = typeSpec.substring(pos + 1, typeSpec.length() - 1);
-	break;
-      default:
-	typeName = typeSpec;
-	break;
+        sb.append(" throws ").append(types[0]);
+        for (int i = 1; i < types.length; i++)
+          sb.append(',').append(types[i]);
       }
-
-    return typeName + arrayPart;
+    return sb.toString();
   }
 
   /**
@@ -330,6 +355,10 @@ extends AccessibleObject implements Member, GenericDeclaration
     return invokeNative(o, args, declaringClass, slot);
   }
 
+  /*
+   * NATIVE HELPERS
+   */
+
   private native Object invokeNative(Object o, Object[] args,
                                      Class declaringClass, int slot)
     throws IllegalAccessException, InvocationTargetException;
@@ -349,6 +378,8 @@ extends AccessibleObject implements Member, GenericDeclaration
   public TypeVariable<Method>[] getTypeParameters()
   {
     String sig = getSignature();
+    if (sig == null)
+      return new TypeVariable[0];
     MethodSignatureParser p = new MethodSignatureParser(this, sig);
     return p.getTypeParameters();
   }
